/*=============================================================================*
 *  Copyright 2006 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/

package org.apache.muse.core;

import java.io.PrintWriter;

import org.w3c.dom.Element;

import org.apache.muse.core.proxy.ProxyHandler;
import org.apache.muse.util.Traceable;
import org.apache.muse.util.messages.Messages;
import org.apache.muse.util.messages.MessagesFactory;
import org.apache.muse.ws.addressing.EndpointReference;
import org.apache.muse.ws.addressing.WsaConstants;
import org.apache.muse.ws.addressing.soap.SimpleSoapClient;
import org.apache.muse.ws.addressing.soap.SoapClient;
import org.apache.muse.ws.addressing.soap.SoapFault;
import org.apache.muse.ws.addressing.soap.SoapMonitor;
import org.apache.muse.ws.addressing.soap.SoapUtils;

/**
 *
 * AbstractResourceClient is a base class for resource clients that are deployed 
 * as doc-literal services using WS-Addressing. It can be used to send raw SOAP 
 * messages to web services, but it mainly serves as the foundation for more 
 * concrete client classes.
 *
 * @author Dan Jemiolo (danj)
 *
 */

public abstract class AbstractResourceClient implements Traceable, SoapMonitor
{
    //
    // Used to lookup all exception messages
    //
    private static Messages _MESSAGES = MessagesFactory.get(AbstractResourceClient.class);
        
    //
    // Where the request is going
    //
    private EndpointReference _destination = null;
    
    //
    // The actual SOAP client used to send requests
    //
    private SoapClient _soapClient = null;
    
    //
    // Where the request is from (the caller's EPR)
    //
    private EndpointReference _source = null;
    
    /**
     * 
     * This is a convenience constructor that calls 
     * this(EndpointReference, EndpointReference) with the WS-A 
     * anonymous EPR as the second parameter (the source EPR).
     * 
     * @see WsaConstants#ANONYMOUS_EPR         
     *
     */
    public AbstractResourceClient(EndpointReference destination)
    {
        this(destination, WsaConstants.ANONYMOUS_EPR);
    }
    
    /**
     * 
     * This is a convenience constructor that calls 
     * this(EndpointReference, EndpointReference, SoapClient) with 
     * an instance of SimpleSoapClient.
     * 
     * @see #AbstractResourceClient(EndpointReference, EndpointReference, SoapClient)
     *
     */
    public AbstractResourceClient(EndpointReference destination, 
                                  EndpointReference source)
    {
        this(destination, source, new SimpleSoapClient());
    }
    
    /**
     * 
     * This is a convenience constructor that calls 
     * this(EndpointReference, EndpointReference, SoapClient) with 
     * the SoapClient provided by the given Environment.
     * 
     * @see #AbstractResourceClient(EndpointReference, EndpointReference, SoapClient)
     * @see Environment#getSoapClient()
     *
     */
    public AbstractResourceClient(EndpointReference destination, 
                                  EndpointReference source, 
                                  Environment environment)
    {
        this(destination, source, environment.getSoapClient());
    }
    
    /**
     *
     * @param destination
     *        The EPR of the resource that will receive requests.
     *  
     * @param source
     *        The EPR of the resource that is sending requests.
     * 
     * @param soapClient
     *        The SOAP client that will be used to package and send requests.
     *
     */
    public AbstractResourceClient(EndpointReference destination, 
                                  EndpointReference source, 
                                  SoapClient soapClient)
    {
        if (destination == null)
            throw new NullPointerException(_MESSAGES.get("NullDestinationEPR"));
        
        if (source == null)
            throw new NullPointerException(_MESSAGES.get("NullSourceEPR"));
        
        if (soapClient == null)
            throw new NullPointerException(_MESSAGES.get("NullSoapClient"));
        
        _destination = destination;
        _source = source;        
        _soapClient = soapClient;
    }
    
    /**
     * 
     * @return The EPR of the resource receiving requests.
     * 
     */
    public EndpointReference getDestination()
    {
        return _destination;
    }

    /**
     * 
     * @return The EPR of the resource receiving requests. This is the same 
     *         value returned by getDestination().
     * 
     */
    public EndpointReference getEndpointReference()
    {
        return getDestination();
    }
    
    /**
     * 
     * @return The SOAP client being used to send requests.
     *
     */
    public SoapClient getSoapClient()
    {
        return _soapClient;
    }

    public int getSoapMonitorPort()
    {
        return getSoapClient().getSoapMonitorPort();
    }
    
    /**
     * 
     * @return The EPR of the resource sending the requests.
     *
     */
    public EndpointReference getSource()
    {
        return _source;
    }
    
    public PrintWriter getTraceWriter()
    {
        return getSoapClient().getTraceWriter();
    }
    
    /**
     * 
     * This is a convenience method that uses the given ProxyHandler to 
     * serialize the parameters (and deserialize the response) before 
     * calling invoke(String, Element).
     * 
     * @see #invoke(String, Element)
     * 
     */
    public Object invoke(ProxyHandler handler, Object[] parameters)
        throws SoapFault
    {
        if (handler == null)
            throw new NullPointerException(_MESSAGES.get("NullProxyHandler"));
        
        String action = handler.getAction();
        Element body = handler.toXML(parameters);
        
        Element response = invoke(action, body);
        
        return handler.fromXML(response);
    }
    
    /**
     * 
     * A generic method invocation mechanism - this can be used by other 
     * proxy methods as well as users in order to send arbitrary requests 
     * to a resource.
     * <br><br>
     * The parameters are wrapped in a SOAP envelope and sent to the 
     * destination using the proxy's SoapClient. If a SOAP Fault is returned, 
     * it is parsed and thrown as a SoapFault object.
     * 
     * @return The SOAP Body from the response message, or null if there was 
     *         no response.
     * 
     */
    public Element invoke(String action, Element soapBody) 
        throws SoapFault 
    {
        return invoke(action, soapBody, new Element[0]);
    }

    
    /**
     * 
     * A generic method invocation mechanism - this can be used by other 
     * proxy methods as well as users in order to send arbitrary requests 
     * to a resource.
     * <br><br>
     * The parameters are wrapped in a SOAP envelope and sent to the 
     * destination using the proxy's SoapClient. If a SOAP Fault is returned, 
     * it is parsed and thrown as a SoapFault object.
     * <br><br>
     * The 'extra headers' provided must be outside of the WS-Addressing 
     * namespace - all WS-A headers are provided by the endpoint references 
     * for the source and destination.
     * 
     * @return The SOAP Body from the response message, or null if there was 
     *         no response.
     * 
     */
    public Element invoke(String action, Element soapBody, Element[] extraHeaders) 
        throws SoapFault 
    {
        //
        // SOAP bodies can technically have more than one element, so we 
        // have to wrap our data in an array
        //
        Element[] body = new Element[] { soapBody };
        
        Element[] response = getSoapClient().send(getSource(), getDestination(), action, body, extraHeaders);
                
        //
        // no response at all is still a valid response
        //
        if (response.length == 0)
            return null;
        
        //
        // faults that aren't network or location-related will be packaged 
        // in the response's SOAP body, so we need to parse and check
        //
        if (SoapUtils.isFault(response[0]))
            throw new SoapFault(response[0]);
        
        return response[0];
    }

    public boolean isUsingSoapMonitor()
    {
        return getSoapClient().isUsingSoapMonitor();
    }
    
    public boolean isUsingTrace()
    {
        return getSoapClient().isUsingTrace();
    }
    
    public void setTrace(boolean trace)
    {
        getSoapClient().setTrace(trace);
    }
    
    public void setTraceWriter(PrintWriter writer)
    {
        getSoapClient().setTraceWriter(writer);
    }

    public void startSoapMonitor(int monitorPort)
    {
        getSoapClient().startSoapMonitor(monitorPort);
    }

    public void stopSoapMonitor()
    {
        getSoapClient().stopSoapMonitor();
    }
}
