/*=============================================================================*
 *  Copyright 2006 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/

package org.apache.muse.core;

import java.util.Collection;
import java.util.logging.Logger;

import org.apache.muse.core.routing.MessageHandler;
import org.apache.muse.ws.addressing.soap.SoapFault;

/**
 * 
 * Capability is the atomic unit of functional definition for a resource 
 * type. A capability class should define a set of data and operations 
 * that are closely-related, but do not necessarily make up the entire 
 * definition of a resource interface. Users aggregate capabilities into 
 * a {@linkplain Resource Resource}, which takes care of delegating 
 * requests to the appropriate capabilities at runtime. Most users will 
 * only have to interact with the pluggable nature of Muse development 
 * through this interface and the &lt;capability/&gt; tag in muse.xml. 
 * 
 * @author Dan Jemiolo (danj)
 * 
 */

public interface Capability extends Initialization, InitializationParameters, Shutdown
{
    /**
     * 
     * @return All of the WS-A Action URIs for which the capability has 
     *         a MessageHandler.
     *         
     * @see #getMessageHandler(String)
     *
     */
    Collection getActions();
    
    /**
     * 
     * @return The unique URI (within the resource type) that defines this 
     *         capability.
     *
     */
    String getCapabilityURI();
    
    /**
     * 
     * @return The capability's access point for things such as the 
     *         WS-Addressing context, file system resources, and more.
     *
     */
    Environment getEnvironment();
    
    /**
     * 
     * @return The JDK logger that writes to the Muse log file.
     *
     */
    Logger getLog();
    
    /**
     * 
     * @param action
     * 
     * @return This method is called as part of the containing Resource's 
     *         method invocation process; it allows the capability to 
     *         specify the handler that will parse the request/response 
     *         for the method.
     *         
     * @see #getActions()
     *
     */
    MessageHandler getMessageHandler(String action);
    
    /**
     * 
     * @return The persistence mechanism for this capability, or null if none 
     *         was defined.
     *
     */
    Persistence getPersistence();
    
    /**
     * 
     * @return The resource that contains this capability instance.
     *
     */
    Resource getResource();
    
    /**
     * 
     * This method is called by the containing Resource after all other 
     * capabilities have been initialized via initialize(). It allows the 
     * Capability to do initialization work that requires the presence of 
     * certain other capabilities. This "post-initialization" state is 
     * necessary because capability definition and creation is 
     * non-deterministic, and users cannot make any assumptions about the 
     * order of those tasks.
     * 
     * @throws SoapFault
     *
     */
    void initializeCompleted()
        throws SoapFault;
    
    /**
     * 
     * This method is called prior to shutdown() and allows the capability 
     * to do any shutdown tasks that require the presence of certain other 
     * capabilities. Once shutdown() is called, the capability cannot be 
     * certain that the other capabilities it depends on have not already 
     * been shutdown as well, so this "pre-shutdown" state gives it this 
     * opportunity.
     *
     * @throws SoapFault
     *
     */
    void prepareShutdown()
        throws SoapFault;
    
    void setCapabilityURI(String capabilityURI);
    
    void setEnvironment(Environment environment);
    
    void setLog(Logger log);
    
    void setMessageHandlers(Collection handlers);
    
    void setPersistence(Persistence persistence);
    
    void setResource(Resource resource);
}
