/*=============================================================================*
 *  Copyright 2006 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/

package org.apache.muse.core;

import java.util.logging.Logger;

import javax.xml.namespace.QName;

import org.w3c.dom.Element;

import org.apache.muse.ws.addressing.EndpointReference;

/**
 * 
 * Resource is an aggregate of Capability objects, providing a common 
 * host for them to share information and sub-components. The 
 * {@linkplain CapabilityContainer CapabilityContainer} interface, while 
 * small, is the most important part of the Resource interface, for it 
 * describes Resource's primary role in the Muse framework.
 * <br><br>
 * This interface also acts as the final stop in the request delegation process, 
 * taking method invocation requests from the 
 * {@linkplain org.apache.muse.core.routing.ResourceRouter ResourceRouter} 
 * and passing them along to the appropriate Capability based on its 
 * knowledge of each Capability's supported operations. This allows the 
 * Capability classes to do their work in a way that is independent of 
 * SOAP request-handling and other plumbing.
 * <br><br>
 * From a remote client perspective, Resource is the atomic unit of 
 * definition, since WSDL 1.1 does not allow resource interfaces to 
 * be broken down into distinct sub-interfaces (capabilities). However, 
 * from a server-side programming model perspective, Resource is not the 
 * most granular level of resource definition - users should focus on creating 
 * {@linkplain Capability capabilities} and use Resource as a means of 
 * communicating between those capabilities.
 *  
 * @author Dan Jemiolo (danj)
 * 
 * @see CapabilityContainer
 * @see Capability
 * @see Capability#getResource()
 *
 */

public interface Resource 
    extends Initialization, InitializationParameters, Shutdown, CapabilityContainer
{    
    /**
     * 
     * @return The unique part of the SOAP endpoint URL that defines this 
     *         resource's type. Every resource type should have a unique 
     *         SOAP endpoint. This value should map to a <em>context-path</em> 
     *         value in muse.
     *
     */
    String getContextPath();
    
    /**
     * 
     * @return The unique EPR of the resource instance.
     *
     */
    EndpointReference getEndpointReference();
    
    /**
     * 
     * @return The resource's access point for things such as the 
     *         WS-Addressing context, file system resources, and more.
     *
     */
    Environment getEnvironment();
    
    /**
     * 
     * @return The JDK logger that writes to the Muse log file.
     *
     */
    Logger getLog();
    
    /**
     * 
     * @return The ResourceManager that stores all of the resource type 
     *         definitions and all current resource instances.
     *
     */
    ResourceManager getResourceManager();
    
    /**
     * 
     * @return The relative path of the WSDL file with the resource's portType.
     *
     */
    String getWsdlPath();
    
    /**
     * 
     * @return The WSDL portType that defines the resource.
     *
     */
    QName getWsdlPortType();
    
    /**
     * 
     * This is a generic method invocation mechanism that can be used to 
     * delegate request handling to a capability or some other component. 
     * It is intended to be used by code that is handling external requests, 
     * and it may not allow the invocation of all methods defined by the type. 
     * Internal clients should make 'normal' Java method calls with the actual 
     * method parameters; the getCapability(String) method provides access 
     * to the Capability objects that make up the resource type, all of which 
     * will have 'normal' Java methods for their operations.
     * 
     * @param soapBody
     *        The parameters for the method, still in their SOAP Body form.
     * 
     * @return The return value of the method, in SOAP Body form. If the 
     *         operation resulted in a fault, the response XML should be 
     *         the fault XML.
     * 
     */
    Element invoke(Element soapBody);
    
    void setContextPath(String contextPath);
    
    void setEndpointReference(EndpointReference epr);
    
    void setEnvironment(Environment environment);
    
    void setLog(Logger log);
    
    void setResourceManager(ResourceManager manager);
    
    void setWsdlPath(String wsdlPath);
    
    void setWsdlPortType(QName wsdlPortType);
}
