/*=============================================================================*
 *  Copyright 2006 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/

package org.apache.muse.util;

import java.util.Date;
import java.util.TimerTask;

import org.apache.muse.util.messages.Messages;
import org.apache.muse.util.messages.MessagesFactory;

/**
 *
 * Timer is a wrapper class for the JDK java.util.Timer that makes 
 * it easy to reschedule timers using the same java.util.TimerTask. 
 * It also provides some extra convenience methods related to scheduling 
 * and execution. All real work is delegated to java.util.Timer.
 *
 * @author Dan Jemiolo (danj)
 *
 */

public class Timer
{
    //
    // Used to lookup all exception messages
    //
    private static Messages _MESSAGES = MessagesFactory.get(Timer.class);
    
    //
    // The task that will be executed when the timer goes off
    //
    private TimerTask _task = null;
    
    //
    // The enclosed timer - we use this for all the real work
    //
    private java.util.Timer _timer = new java.util.Timer();
    
    //
    // The time when this timer will execute its task
    //
    private Date _whenToFire = null;
    
    /**
     * 
     * Cancels the timer. If the timer was not scheduled, this is a no-op. 
     * If it <b>was</b> scheduled, its task will not be executed and 
     * subsequent calls to getScheduledTime() will return null.
     * <br><br>
     * To reschedule a timer after it has been cancelled, you must use the 
     * reschedule(TimerTask, Date) method - you cannot just call schedule(TimerTask, Date).
     * 
     */
    public final void cancel()
    {
        _timer.cancel();
        _whenToFire = null;
    }
    
    /**
     * 
     * Executes the timer's task whether the timer is scheduled or not. The 
     * implementation first tries to cancel the timer, then executes the 
     * task directly (via TimerTask.run()).
     *
     */
    public final void executeNow()
    {
        cancel();
        _task.run();
    }
    
    /**
     * 
     * @return The time when this timer is set to fire, or null if it is 
     *         not scheduled.
     *
     */
    public final Date getScheduledTime()
    {
        return _whenToFire;
    }
    
    /**
     * 
     * @return The task that will be executed if and when the timer is fired.
     *
     */
    public final TimerTask getTimerTask()
    {
        return _task;
    }
    
    /**
     *
     * @return True, if the timer is scheduled <b>and</b> the current time 
     *        is later than the schedule time.
     *
     */
    public final boolean isPastScheduledTime()
    {
        return _whenToFire != null && _whenToFire.before(new Date());
    }
    
    /**
     * 
     * Cancels the timer (if it is currently scheduled) and reschedules it 
     * for the given time.
     *
     * @param task
     *        The code to run when the timer is up.
     *        
     * @param whenToFire
     *        The new fire time for the timer.
     *
     */
    public final void reschedule(TimerTask task, Date whenToFire)
    {
        if (task == null)
            throw new NullPointerException(_MESSAGES.get("NullTimerTask"));
        
        if (whenToFire == null)
            throw new NullPointerException(_MESSAGES.get("NullFireTime"));
        
        //
        // you can't reschedule a java.util.Timer, so we just kill it 
        // and make a new one
        //
        cancel();
        _timer = new java.util.Timer();
        
        schedule(task, whenToFire);
    }
    
    /**
     * 
     * Sets the timer to fire at the given time, which will results in the 
     * timer's TimerTask being executed. If the given time is in the past, 
     * the timer will execute immediately (or as soon as its thread is given 
     * control).
     * 
     * @param task
     *        The code to run when the timer is up.
     *
     * @param whenToFire
     *        The time when the timer should fire.
     *
     */
    public final void schedule(TimerTask task, Date whenToFire)
    {
        if (task == null)
            throw new NullPointerException(_MESSAGES.get("NullTimerTask"));
        
        if (whenToFire == null)
            throw new NullPointerException(_MESSAGES.get("NullFireTime"));
        
        _task = task;
        _whenToFire = whenToFire;
        
        _timer.schedule(_task, _whenToFire);
    }
}
