/*=============================================================================*
 *  Copyright 2006 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/

package org.apache.muse.util.messages;

import java.util.Locale;
import java.util.ResourceBundle;

/**
 *
 * Messages is a wrapper for a java.util.ResourceBundle that provides a 
 * search-and-replace mechanism for filling in message context. Users 
 * can create properties files with messages that include <b>XXX</b> 
 * placeholders and then fill in the values of these placeholders at 
 * runtime. This allows for as much message externalization as possible 
 * while still making it easy to provide runtime-specific information.  
 *
 * @author Dan Jemiolo (danj)
 * 
 * @see MessagesFactory
 * @see ResourceBundle
 *
 */

public class Messages
{
    private static final String _PLACE_HOLDER = "XXX";
    
    //
    // Used for requests that don't have any placeholders
    //
    private Object[] _EMPTY = new Object[0];
    
    //
    // The underlying bundle that stores all the properties/messages
    //
    private ResourceBundle _messages = null;
    
    /**
     * 
     * Creates a new message collection using the given resource bundle. 
     * The bundle (properties file) may contain <b>XXX</b> placeholders 
     * that indicate where values will be substituted at runtime.
     *
     * @param bundleName
     *        The qualified name of the resource bundle to parse.
     *
     */
    public Messages(String bundleName)
    {
        _messages = ResourceBundle.getBundle(bundleName);        
    }
    
    /**
     * 
     * Creates a new message collection using the given resource bundle and 
     * classloader. The bundle (properties file) may contain <b>XXX</b> 
     * placeholders that indicate where values will be substituted at runtime. 
     * This constructor is used by dependent bundles so that their resources 
     * can be found by the resource bundle.
     *
     * @param bundleName
     *        The qualified name of the resource bundle to parse.
     * 
     * @param loader
     *        The class loader that has the bundle file in its classpath.
     *
     */
    public Messages(String bundleName, ClassLoader loader) 
    {
    	_messages = ResourceBundle.getBundle(bundleName, Locale.getDefault(), loader);
    }
    
    /**
     * 
     * @param name
     *        The name of the property/message to lookup.
     * 
     * @return The message value, with a prefix that identifies the message name.
     *
     */
    public String get(String name)
    {
        return get(name, _EMPTY);
    }
    
    /**
     * 
     * @param name
     *        The name of the property/message to lookup.
     * 
     * @param includeID
     *        True if you want the message to include [ID = "MessageName"] 
     *        for debugging purposes.
     * 
     * @return The message value, with a prefix that identifies the message name.
     *
     */
    public String get(String name, boolean includeID)
    {
        return get(name, _EMPTY, includeID);
    }
    
    /**
     * 
     * Retrieves the value specified and replaces any placeholders (<b>XXX</b> 
     * values) with the values in the given array. The filler values are 
     * serialized using Object.toString().
     * <br><br>
     * If the number of filler values is less than the number of placeholders, 
     * the placeholders will be left in the string. If the number of filler 
     * values is greater, the leftover values will simply not be used.
     * <br><br>
     * <b>NOTE:</b> If any of the filler values contains a regex, the 
     * replacement mechanism (String.replaceFirst()) may try to perform 
     * grouping or other regex functions, causing the replacement to fail. 
     * If this method throws a RuntimeException that has to do with regular 
     * expressions or string replacement, the problem is that one of your 
     * filler values has regex symbols in it.
     * 
     * @param name
     *        The name of the property/message to lookup.
     * 
     * @param filler
     *        The values to substitute for the placeholders (if any).
     * 
     * @return The message value, with the given filler values substituted for 
     *         the placeholders and a prefix that identifies the message name.
     *
     */
    public String get(String name, Object[] filler)
    {
        return get(name, filler, true);
    }
    
    /**
     * 
     * Retrieves the value specified and replaces any placeholders (<b>XXX</b> 
     * values) with the values in the given array. The filler values are 
     * serialized using Object.toString().
     * <br><br>
     * If the number of filler values is less than the number of placeholders, 
     * the placeholders will be left in the string. If the number of filler 
     * values is greater, the leftover values will simply not be used.
     * <br><br>
     * <b>NOTE:</b> If any of the filler values contains a regex, the 
     * replacement mechanism (String.replaceFirst()) may try to perform 
     * grouping or other regex functions, causing the replacement to fail. 
     * If this method throws a RuntimeException that has to do with regular 
     * expressions or string replacement, the problem is that one of your 
     * filler values has regex symbols in it.
     * 
     * @param name
     *        The name of the property/message to lookup.
     * 
     * @param filler
     *        The values to substitute for the placeholders (if any).
     * 
     * @param includeID
     *        True if you want the message to include [ID = "MessageName"] 
     *        for debugging purposes.
     * 
     * @return The message value, with the given filler values substituted for 
     *         the placeholders and a prefix that identifies the message name.
     *
     */
    public String get(String name, Object[] filler, boolean includeID)
    {
        String message = _messages.getString(name);
        
        //
        // this is slow because it's an n^2 operation
        //
        for (int n = 0; n < filler.length; ++n)
        {
            //
            // we replace all \ in the user's text because they get 
            // stripped out in XML/SOAP messages
            //
            String next = filler[n] == null ? "null" : filler[n].toString();
            next = next.replace('\\', '/');
            
            //
            // NOTE: this will throw a runtime exception if the value 
            //       of the "next" filler item contains a regex that 
            //       causes the pattern matcher to try and group results 
            //       together.
            //
            message = message.replaceFirst(_PLACE_HOLDER, next);
        }
        
        //
        // build up the error text - include the bundle ID for debugging
        //
        int length = name.length() + message.length() + 16;
        StringBuffer buffer = new StringBuffer(length);
        
        if (includeID)
        {
            buffer.append("[ID = '");
            buffer.append(name);
            buffer.append("'] ");
        }
        
        buffer.append(message);
        return buffer.toString();
    }
}
