/*=============================================================================*
 *  Copyright 2006 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/

package org.apache.muse.ws.addressing.soap;

import org.w3c.dom.Element;

import org.apache.muse.util.Traceable;
import org.apache.muse.ws.addressing.EndpointReference;
import org.apache.muse.ws.addressing.soap.SoapMonitor;

/**
 *
 * SoapClient is a generic interface for a common set of SOAP utilities that 
 * may be implemented with different APIs from platform to platform. It allows 
 * users to send messages without putting API-specific references in their code.
 *
 * @author Dan Jemiolo (danj)
 *
 */

public interface SoapClient extends SoapMonitor, Traceable
{
    /**
     * 
     * Sends a SOAP envelope with the given body to the given destination. 
     * The SOAP header will include WS-A headers, one of which will be the 
     * WS-A Action specified in the third parameter. This method is not 
     * asynchronous - it will wait for a response and return the contents 
     * of the SOAP body.
     *
     * @param src
     *        The EPR of the service sending the message.
     * 
     * @param dest
     *        The EPR of the service that will receive the message.
     * 
     * @param wsaAction
     *        The WS-A Action that specifies the operation to be invoked on 
     *        the destination service.
     * 
     * @param body
     *        The contents of the SOAP body.
     * 
     * @return The contents of the SOAP body in the response message. If 
     *         the request resulted in a fault, the contents will include 
     *         the fault XML. Note that such faults are not related to 
     *         basic networking/SOAP faults.
     *
     */
    Element[] send(EndpointReference src, EndpointReference dest, String wsaAction, Element[] body);

    /**
     * 
     * Sends a SOAP envelope with the given body to the given destination. 
     * The SOAP header will include WS-A headers, one of which will be the 
     * WS-A Action specified in the third parameter. This method is not 
     * asynchronous - it will wait for a response and return the contents 
     * of the SOAP body.
     *
     * @param src
     *        The EPR of the service sending the message.
     * 
     * @param dest
     *        The EPR of the service that will receive the message.
     * 
     * @param wsaAction
     *        The WS-A Action that specifies the operation to be invoked on 
     *        the destination service.
     * 
     * @param body
     *        The contents of the SOAP body.
     *        
     * @param extraHeaders
     *        SOAP header elements that should be added to the outgoing message. 
     *        These headers <b>must not</b> use the WS-Addressing namespace.
     * 
     * @return The contents of the SOAP body in the response message. If 
     *         the request resulted in a fault, the contents will include 
     *         the fault XML. Note that such faults are not related to 
     *         basic networking/SOAP faults.
     *
     */
    Element[] send(EndpointReference src, EndpointReference dest, String wsaAction, Element[] body, Element[] extraHeaders);
}
