/*=============================================================================*
 *  Copyright 2006 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/

package org.apache.muse.ws.dm.muws;

import java.util.Date;

import javax.xml.namespace.QName;

import org.apache.muse.ws.resource.WsResource;
import org.apache.muse.ws.resource.basefaults.BaseFault;

/**
 * 
 * Metric is an interface that describes the metadata surrounding resource 
 * properties that are metrics, as defined by the WSDM Metrics capability. 
 * It does not represent the Metrics capability itself - that is handled by 
 * the {@linkplain Metrics Metrics interface}. Implementations of this type 
 * can be used to store and update the metadata about a metric property as 
 * read and write operations are performed against it.
 *
 * @author Dan Jemiolo (danj)
 *
 */

public interface Metric
{
    int COUNTER = 1;

    int GAUGE = 2;

    int UNKNOWN = 3;

    int INTERVAL = 4;

    int POINT_IN_TIME = 5;

    int SINCE_RESET = 6;

    int ON_CHANGE = 7;

    int ON_DEMAND = 8;

    int PERIODIC = 9;

    String getDuration();

    String getGroup();

    long getInterval();

    String getIntervalString();

    Date getLastUpdated();
    
    /**
     * 
     * @return The name of the resource property that this metric represents.
     *
     */
    QName getName();

    Date getResetAt();
    
    /**
     * 
     * @return The resource instance that contains the resource property that 
     *         this metric represents.
     *
     */
    WsResource getWsResource();
    
    /**
     * 
     * @return True if no calls to update() have been made since the last 
     *         call to reset().
     *
     */
    boolean hasBeenReset();

    boolean isCounter();

    boolean isGauge();

    boolean isInterval();

    boolean isOnChange();

    boolean isOnDemand();

    boolean isPeriodic();

    boolean isPointInTime();

    boolean isSinceReset();
    
    /**
     * 
     * Updates the resource property via WSRP and then adjusts the metric 
     * metadata to reflect the change.
     * 
     * @param value
     *        The value to be provided to the WSRP implementation when updating 
     *        the resource property.
     * 
     */
    void reset(Object value) 
        throws BaseFault;
    
    /**
     * 
     * Notifies the metric metadata container that an update to the resource 
     * property occurred, so it should adjust the metadata values to reflect 
     * this change.
     *
     */
    void update();
}
