/*=============================================================================*
 *  Copyright 2006 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/

package org.apache.muse.ws.notification.impl;

import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.Set;

import org.apache.muse.ws.notification.NotificationMessage;
import org.apache.muse.ws.notification.PullPointDataStore;

/**
 * 
 * SimplePullPointDataStore is an in-memory container for messages received by 
 * a pullpoint resource. It stores messages in a linked hash set so that they 
 * can be retrieved in order in O(1) time. There is no maximum size for the 
 * collection, so footprint of the application could easily get out-of-hand 
 * unless the user provides additional logic to handle this (although it is 
 * not clear what else could be done aside from dropping messages). A better 
 * implementation would rely on a persistent data store and asynchronous 
 * additions and deletions.
 *
 * @author Dan Jemiolo (danj)
 *
 */

public class SimplePullPointDataStore implements PullPointDataStore
{
    private boolean _hasBeenInitialized = false;
    
    private boolean _hasBeenShutdown = false;
    
    //
    // use a linked hash set so that insertion/deletion is O(1) while still 
    // allowing us to maintain order for getMessages()
    //
    private Set _messages = new LinkedHashSet();

    public synchronized void addMessage(NotificationMessage message)
    {
        _messages.add(message);
    }

    public synchronized NotificationMessage[] getMessages(int maxNumber)
    {
        int currentSize = _messages.size();
        
        //
        // make sure we don't take more than we have
        //
        if (currentSize < maxNumber)
            maxNumber = currentSize;
        
        NotificationMessage[] results = new NotificationMessage[maxNumber];
        Iterator i = _messages.iterator();
        
        //
        // delete from the list so messages aren't returned twice
        //
        for (int n = 0; n < maxNumber; ++n)
        {
            results[n] = (NotificationMessage)i.next();
            i.remove();
        }
        
        return results;
    }

    public boolean hasBeenInitialized()
    {
        return _hasBeenInitialized;
    }

    public boolean hasBeenShutdown()
    {
        return _hasBeenShutdown;
    }

    public void initialize()
    {
        //
        // no additional initialization tasks needed
        //
        _hasBeenInitialized = true;
    }

    public void shutdown()
    {
        //
        // no shutdown tasks - messages are not persisted
        //
        _hasBeenShutdown = true;
    }
}
