/*=============================================================================*
 *  Copyright 2006 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/

package org.apache.muse.ws.resource.properties.schema;

import java.util.Collection;

import javax.xml.namespace.QName;

import org.apache.muse.ws.resource.WsResourceCapability;
import org.apache.muse.ws.resource.basefaults.BaseFault;

/**
 * 
 * ResourcePropertiesSchemaValidation is a set of methods for WSRP 
 * implementations that provide schema validation and a capability 
 * delegation model.
 *
 * @author Dan Jemiolo (danj)
 *
 */

public interface ResourcePropertiesSchemaValidation
{
    /**
     * 
     * Registers the capability with the schema so that users of the schema 
     * can determine which capability is responsible for which resource properties.
     * 
     * @param capability
     *
     */
    void addCapability(WsResourceCapability capability);
    
    /**
     * 
     * @param qname
     *        The name of the resource property whose capability is being queried.
     * 
     * @return The capability that defines the given property, or null if the 
     *         property does not exist.
     *
     */
    WsResourceCapability getCapability(QName qname);
    
    /**
     * 
     * @return All of the resource properties defined by the schema.
     *
     */
    Collection getPropertyNames();
    
    /**
     * 
     * @return The XSD for this WS-RP document.
     *
     */
    ResourcePropertiesSchema getSchema();
    
    /**
     * 
     * @param qname
     *        The name of a resource property to search the schema for.
     * 
     * @return True if there is a schema definition with the given name.
     *
     */
    boolean hasPropertyDefinition(QName qname);
    
    /**
     * 
     * Applies the given XMLSchema definitions to this WS-RP document. This 
     * schema will be used for all subsequent validations, including calls 
     * to validateSchema() and SetResourceProperties-like operations.
     * <br><br>
     * Note that this method does <b>not</b> perform validation.
     *
     * @param schema
     *        The XSD properties definition for this WS-RP document.
     * 
     * @see #validateSchema()
     * 
     */
    void setSchema(ResourcePropertiesSchema schema);
    
    /**
     * 
     * Iterates through every property in the WS-RP document's XMLSchema and 
     * verifies current document is valid. This method should be used 
     * sparingly, as it will validate every property; for cases such as 
     * SetResourceProperties, the implementation of the operations should 
     * validate the individual property being targeted, not the whole document.
     *
     * @throws BaseFault
     *         <ul>
     *         <li>If the number of instances of a property is below the 
     *         required minimum.</li>
     *         <li>If the number of instances of a property is above the 
     *         required maximum.</li>
     *         <li>If an instance of the property is null (the equivalent of 
     *         an empty XML element) but the property is not nillable.</li>
     *         </ul>
     *
     */
    void validateSchema()
        throws BaseFault;
}
