/*=============================================================================*
 *  Copyright 2006 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/

package org.apache.muse.ws.resource.properties.set;

import javax.xml.namespace.QName;

import org.w3c.dom.Element;

import org.apache.muse.ws.resource.basefaults.BaseFault;

/**
 * 
 * SetOperationFactory is a component that allows users to create executable 
 * commands for a WSRP document by providing either the values of the command 
 * or an XML fragment that represents it. All commands are compliant with the 
 * WSRP v1.2 SetResourceProperties spec definition.
 *
 * @author Dan Jemiolo (danj)
 *
 */
public interface SetOperationFactory
{
    /**
     * 
     * This is a convenience method that creates a new SetRequest object 
     * an populates it with a DeleteRequest for the given property.
     *
     * @param qname
     *        The name of the property to delete.
     * 
     */
    SetRequest createDelete(QName qname);

    /**
     * 
     * This is a convenience method that creates a new SetRequest object 
     * an populates it with a InsertRequest for the given property.
     *
     * @param qname
     *        The name of the property to insert.
     * 
     * @param values
     *        The initial values for each instance of the property. This 
     *        array should not be null, nor empty.
     * 
     */
    SetRequest createInsert(QName qname, Object[] values);

    /**
     * 
     * Parses the given XML fragment according to the WS-RP spec for 
     * SetResourceProperties requests.
     *
     * @param request
     *        An XML fragment containing a SetResourceProperties request.
     * 
     * @return A SimpleSetRequest that contains one AbstractSetRequestComponent for each 
     *         operation defined in the request. This object can be used 
     *         to execute the request against a WS-RP container.
     * 
     * @throws BaseFault
     *         <ul>
     *         <li>If the request is empty (contains no operations).</li>
     *         <li>If one of the operation types is invalid (only Insert, 
     *         Delete, and Update are allowed).</li>
     *         <li>If one of the operations is not properly formatted.</li>
     *         </ul>
     *
     */
    SetRequest createSet(Element request)
        throws BaseFault;

    /**
     * 
     * This is a convenience method that creates a new SetRequest object 
     * an populates it with a UpdateRequest for the given property.
     *
     * @param qname
     *        The name of the property to update.
     * 
     * @param value
     *        The value that will be copied to each instance of the property.
     * 
     */
    SetRequest createUpdate(QName qname, Object[] values);
}
