/*=============================================================================*
 *  Copyright 2006 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/

package org.apache.muse.ws.resource.properties.set;

import javax.xml.namespace.QName;

import org.apache.muse.ws.resource.basefaults.BaseFault;

/**
 *
 * SetResourcePropertyPermissions defines a security mechanism that can be used 
 * to limit the use of these methods to internal callers, and/or to differentiate 
 * between internal and external callers so that internal requests can have 
 * more freedom in their modifications. A resource may need to modify properties 
 * in a way that is not allowed from an external client; an example would be 
 * a read-only property such as "free disk space" which changes frequently 
 * but is not directly write-able by clients. It also allows users to define 
 * specialized methods for accessing certain properties and disallow access 
 * through the generic WS-RP methods.
 * <br><br>
 * The security token is meant to be a simple unique value that cannot be 
 * easily generated by an external caller; for many implementations, the use 
 * of a simple Object reference will suffice, since the JVM will never produce 
 * two references with the same address. If code is written that uses the 
 * security token version of these methods, the calls will not succeed unless 
 * the token matches the document's internal token.
 *
 * @author Dan Jemiolo (danj)
 *
 */
public interface SetResourcePropertiesPermissions
{
    /**
     * 
     * Removes <b>all</b> properties with the given name. This call will not 
     * complete if the given security token doesn't match the document's token 
     * <b>and</b> the caller is trying to delete an immutable property.
     *
     * @param qname
     *        The name of the property to delete.
     * 
     * @param securityToken
     *        The token used to verify permission to call this method.
     * 
     * @throws BaseFault
     *         <ul>
     *         <li>If there were zero instances of the property named.</li>
     *         <li>If performing the delete would result in the number of 
     *         property instances being below the schema's minimum value.</li>
     *         <li>If the property is read-only or insert-only and the 
     *         security token is not valid.</li>
     *         <li>If one of the PropertyChangeListeners assigned to this 
     *         property throws an exception while reacting to the deletion.</li>
     *         </ul>
     *
     */
    void deleteResourceProperty(QName qname, Object securityToken) 
        throws BaseFault;

    /**
     * 
     * Returns the security token that can be used by users of the WS-RP 
     * document to verify that they are internal callers with permission 
     * to circumvent published restrictions on property changes, etc. The 
     * security token system may be used to allow modification of read-only 
     * properties, modification to properties that are not supposed to be 
     * set with WS-RP's generic SetResourceProperties, and other special 
     * behavior.
     * <br><br>
     * The token should be given to internal components that will need 
     * special permissions, but it should not be used by components that 
     * are handling external requests directly.
     *
     * @return The valid security token for the WS-RP document.
     *
     */
    Object getSecurityToken();

    /**
     * 
     * Creates a new instance of the given property for each value specified 
     * in the array of values. This call will not complete if the given 
     * security token doesn't match the document's token <b>and</b> the 
     * caller is trying to insert an immutable property.
     *
     * @param qname
     *        The name of the property instances to create.
     * 
     * @param values
     *        The set of values that will be assigned to the new property 
     *        instances. Each value will be created once.
     * 
     * @param securityToken
     *        The token used to verify permission to call this method.
     * 
     * @throws BaseFault
     *         <ul>
     *         <li>If the property is not defined in the document's schema.</li>
     *         <li>If adding the new instances would create more than the 
     *         maximum defined by the schema.</li>
     *         <li>If the property is read-only and the security token is not 
     *         valid.</li>
     *         <li>If one of the PropertyChangeListeners assigned to this 
     *         property throws an exception while reacting to the insertion.</li>
     *         <li>If the container tries to create or manipulate the Topic 
     *         for the property and fails.</li>
     *         </ul>
     *
     */
    void insertResourceProperty(QName qname, Object[] values, Object securityToken)
        throws BaseFault;

    /**
     * 
     * Updates <b>all</b> instances of the given property using the given 
     * values; all previous values will be removed in lieu of these new 
     * values. This call will not complete if the given security token 
     * doesn't match the document's token <b>and</b> the caller is trying 
     * to update an immutable property.
     *
     * @param qname
     *        The name of the property to update.
     * 
     * @param values
     *        The values to assign to the property.
     * 
     * @param securityToken
     *        The token used to verify permission to call this method.
     * 
     * @throws BaseFault
     *         <ul>
     *         <li>If the property is not defined in the document's schema.</li>
     *         <li>If a new value is null and the property is not nillable.</li>
     *         <li>If the property is read-only and the security token is not 
     *         valid.</li>
     *         <li>If one of the PropertyChangeListeners assigned to this 
     *         property throws an exception while reacting to the update.</li>
     *         </ul>
     *
     */
    void updateResourceProperty(QName qname, Object[] values, Object securityToken)
        throws BaseFault;
}
