/*=============================================================================*
 *  Copyright 2006 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/

package org.apache.muse.ws.resource.lifetime.impl;

import java.text.ParseException;
import java.util.Date;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import org.apache.muse.util.messages.Messages;
import org.apache.muse.util.messages.MessagesFactory;
import org.apache.muse.util.xml.XmlSerializable;
import org.apache.muse.util.xml.XmlUtils;
import org.apache.muse.util.xml.XsdUtils;
import org.apache.muse.ws.resource.lifetime.WsrlConstants;
import org.apache.muse.ws.resource.lifetime.faults.UnableToSetTerminationTimeFault;

/**
 *
 * SetTerminationTime is a serializer/deserializer for the WS-ImmediateTermination 
 * SetTerminationTime operation's request content.
 *
 * @author Dan Jemiolo (danj)
 *
 */

public class SetTerminationTime implements XmlSerializable
{
    //
    // Used to lookup all exception messages
    //
    private Messages _MESSAGES = MessagesFactory.get(SetTerminationTime.class);
    
    //
    // The new wsrl:TerminationTime (could be null)
    //
    private Date _time = null;
    
    public SetTerminationTime(Date time)
    {
        _time = time;
    }
    
    public SetTerminationTime(Element request) 
        throws UnableToSetTerminationTimeFault
    {
        if (request == null)
            throw new NullPointerException(_MESSAGES.get("NullRequestElement"));
        
        boolean isDuration = true;
        String timeString = XmlUtils.getElementText(request, WsrlConstants.REQUESTED_DURATION_QNAME);
        
        //
        // no duration, try dateTime
        //
        if (timeString == null)
        {
            isDuration = false;
            timeString = XmlUtils.getElementText(request, WsrlConstants.REQUESTED_TIME_QNAME);
        }

        //
        // the time could be null - this means there is no 
        // scheduled termination any more
        //
        if (timeString != null)
        {
            try
            {
                if (isDuration)
                {
                    Date rightNow = new Date();
                    long duration = XsdUtils.getDuration(timeString);
                    long theFuture = rightNow.getTime() + duration;
                    _time = new Date(theFuture);
                }
                
                else
                    _time = XsdUtils.getLocalTime(timeString);
            }
            
            catch (ParseException error)
            {
                Object[] filler = { timeString, error.getMessage() };
                throw new UnableToSetTerminationTimeFault(_MESSAGES.get("InvalidTimeValue", filler));
            }
        }
    }
    
    public Date getTerminationTime()
    {
        return _time;
    }
    
    public String toString()
    {
        return XmlUtils.toString(toXML(), false);
    }
    
    public Element toXML()
    {
        return toXML(XmlUtils.EMPTY_DOC);
    }
    
    public Element toXML(Document doc)
    {
        if (doc == null)
            throw new NullPointerException(_MESSAGES.get("NullDocument"));
        
        Element root = XmlUtils.createElement(doc, WsrlConstants.SET_TERMINATION_QNAME);
        XmlUtils.setElement(root, WsrlConstants.REQUESTED_TIME_QNAME, getTerminationTime());
        
        return root;        
    }
}
